=== modified file 'bzrlib/tests/branch_implementations/test_branch.py'
--- bzrlib/tests/branch_implementations/test_branch.py	2007-03-26 01:02:53 +0000
+++ bzrlib/tests/branch_implementations/test_branch.py	2007-03-26 01:41:52 +0000
@@ -616,9 +616,27 @@
 
 
 class TestRevisionHistoryCaching(TestCaseWithBranch):
+    """Tests for the caching of branch revision_history.
+
+    When locked, branches should avoid regenerating or rereading
+    revision_history by caching the last value of it.  This is safe because
+    the branch is locked, so nothing can change the revision_history
+    unexpectedly.
+
+    When not locked, obviously the revision_history will need to be regenerated
+    or reread each time.
+
+    We test if revision_history is using the cache by instrumenting the branch's
+    _gen_revision_history method, which is called by Branch.revision_history if
+    the branch does not have a cache of the revision history.
+    """
 
     def get_instrumented_branch(self):
-        # Get a branch and monkey patch it to log calls to _gen_revision_history
+        """Get a branch and monkey patch it to log calls to
+        _gen_revision_history.
+
+        :returns: a tuple of (the branch, list that calls will be logged to)
+        """
         branch = self.get_branch()
         calls = []
         real_gen_revision_history = branch._gen_revision_history
@@ -634,8 +652,6 @@
         """
         branch, calls = self.get_instrumented_branch()
         # Repeatedly call revision_history.
-        # _gen_revision_history will called each time, because the branch is not
-        # locked, so the history might have changed between calls.
         branch.revision_history()
         branch.revision_history()
         self.assertEqual(
@@ -647,7 +663,6 @@
         """
         branch, calls = self.get_instrumented_branch()
         # Lock the branch, then repeatedly call revision_history.
-        # _gen_revision_history will only be called once.
         branch.lock_read()
         try:
             branch.revision_history()
@@ -656,6 +671,51 @@
         finally:
             branch.unlock()
 
+    def test_set_revision_history_when_locked(self):
+        """When the branch is locked, calling set_revision_history should cache
+        the revision history so that a later call to revision_history will not
+        need to call _gen_revision_history.
+        """
+        branch, calls = self.get_instrumented_branch()
+        # Lock the branch, set the revision history, then repeatedly call
+        # revision_history.
+        branch.lock_write()
+        branch.set_revision_history([])
+        try:
+            branch.revision_history()
+            self.assertEqual([], calls)
+        finally:
+            branch.unlock()
+
+    def test_set_revision_history_when_unlocked(self):
+        """When the branch is not locked, calling set_revision_history will not
+        cause the revision history to be cached.
+        """
+        branch, calls = self.get_instrumented_branch()
+        # Lock the branch, set the revision history, then repeatedly call
+        # revision_history.
+        branch.set_revision_history([])
+        branch.revision_history()
+        self.assertEqual(['_gen_revision_history'], calls)
+
+    def test_set_last_revision_info_when_locked(self):
+        """When the branch is locked, calling set_last_revision_info should
+        cache the last revision info so that a later call to last_revision_info
+        will not need the revision_history.  Thus the branch will not to call
+        _gen_revision_history in this situation.
+        """
+        branch, calls = self.get_instrumented_branch()
+        # Lock the branch, set the revision history, then repeatedly call
+        # revision_history.
+        branch.lock_write()
+        branch.set_last_revision_info(0, None)
+        del calls[:]
+        try:
+            branch.last_revision_info()
+            self.assertEqual([], calls)
+        finally:
+            branch.unlock()
+
 
 class TestBranchPushLocations(TestCaseWithBranch):
 

